﻿using System;
using EPiServer.Core;
using EPiServer.Common;
using EPiServer.Web;

namespace EPiServer.Templates.RelatePlus.Templates.PageEntity
{
    public abstract class PageEntity : FrameworkEntityBase
    {
        #region Member Variables
        private readonly string[] _casheKey;
        private PageReference _pageLink = PageReference.EmptyReference;
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="PageEntity"/> class.
        /// </summary>
        /// <param name="pageReference">The page reference.</param>
        public PageEntity(PageReference pageReference)
            : this(pageReference, PageEntityTypes.Undefined)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PageEntity"/> class.
        /// </summary>
        /// <param name="pageReference">The page reference.</param>
        /// <param name="entityType">Type of the page.</param>
        public PageEntity(PageReference pageReference,PageEntityTypes entityType)
            : this(PermanentLinkUtility.FindGuid(pageReference), entityType)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PageEntity"/> class.
        /// </summary>
        /// <param name="pageGuid">The page GUID.</param>
        public PageEntity(Guid pageGuid)
            : this(pageGuid, PageEntityTypes.Undefined)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PageEntity"/> class.
        /// </summary>
        /// <param name="pageGuid">The page GUID.</param>
        /// <param name="entityType">Type of the entity.</param>
        public PageEntity(Guid pageGuid, PageEntityTypes entityType)
            : this(-1, pageGuid, entityType)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PageEntity"/> class.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="pageGuid">The page GUID.</param>
        /// <param name="entityType">Type of the entity.</param>
        protected internal PageEntity(int id, Guid pageGuid, PageEntityTypes entityType)
            : base(id)
        {
            PageGUID = pageGuid;            
            EntityType = entityType;
            _casheKey = ConstructCacheKey(id);                      
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PageEntity"/> class.
        /// </summary>
        protected internal PageEntity()
            : base(-1)
        {
        }
        #endregion

        #region Properties
        public override string[] CacheKey
        {
            get { return _casheKey; }
        }

        /// <summary>
        /// Gets the page link.
        /// </summary>
        /// <value>The page link.</value>
        public PageReference PageLink 
        {
            get
            {
                if (PageReference.IsNullOrEmpty(_pageLink))
                {
                    _pageLink = PermanentLinkUtility.FindPageReference(PageGUID);
                }
                return _pageLink;
            }
        }
        public Guid PageGUID { get; private set; }
        public PageEntityTypes EntityType { get; set; }
        #endregion

        #region Internal Methods
        internal static string[] ConstructCacheKey(int id)
        {
            return new [] { "PageEntity", "PageEntity", id.ToString() };
        }

        internal static string[] ConstructCacheKey(Guid guid)
        {
            return new [] { "PageEntity", "PageEntity", guid.ToString() };
        }
        #endregion

    }
}